// Библиотеки отвечающие за режим сна
#include <avr/sleep.h>
#include <avr/power.h>
// Библиотеки для работы с дисплеем 5110
#include <Adafruit_GFX.h>
#include <Adafruit_PCD8544.h>
// Библиотека работы с устройствами по I2C
#include <Wire.h> 
// Библиотека работы с TSL2561
#include <Adafruit_Sensor.h>
#include <Adafruit_TSL2561_U.h>
 
#define MAX_UNITS  84 

// pin 7 - Serial clock out (SCLK)
// pin 6 - Serial data out (DIN)
// pin 5 - Data/Command select (D/C)
// pin 4 - LCD chip select (CS)
// pin 3 - LCD reset (RST)
Adafruit_PCD8544 display = Adafruit_PCD8544(3, 4, 5, 6, 7);
Adafruit_TSL2561_Unified tsl = Adafruit_TSL2561_Unified(TSL2561_ADDR_FLOAT, 12345);

boolean DebugFlag = true;


int key_count          = 0; 
unsigned long key_time = 0;
boolean key_press      = false;
boolean power_on       = false;

int key_count2          = 0; 
unsigned long key_time2 = 0;
boolean key_press2      = false;
int key_mode2           = 1;


volatile boolean int_pwr_up  = false;
volatile boolean int_key     = false;

/**
* Функция определения напряжения питания устройства
*/
static int vccRead(byte us =250) {
  ADMUX = 1<<REFS0; // опорное напряжение - Vcc
  ADMUX |= 0x0E;    // объект измерения - внутренний источник
                    // стабилизированного напряжения 1.1В
  delayMicroseconds(us);
 
  ADCSRA |= 1<<ADSC;         // запуск АЦ-преобразования
  while(ADCSRA & (1<<ADSC)); // и ожидание его завершения
  word x = ADC;
  return x ? (1100L * 1023) / x : -1;
}


/**
* Функция инициализации при включении питания 
*/
void PowerOn(){
  digitalWrite(8,HIGH);   //  on
  digitalWrite(9,HIGH);   // on
  digitalWrite(13,HIGH);  // power LED on
  display.begin();
  Wire.begin();
  tsl.begin();
  tsl.enableAutoGain(true);          /* Auto-gain ... switches automatically between 1x and 16x */
  tsl.setIntegrationTime(TSL2561_INTEGRATIONTIME_13MS);      /* fast but low resolution */
  display.setContrast(40);
}

/**
* Функция выключение всех устройств при переходе в POWER_DOWN моде
*/
void PowerOff(){
// Отключаем все выходы 3-19
  for( int i=3; i<20; i++ ){
     pinMode(i,INPUT_PULLUP); 
     digitalWrite(i,LOW);  
  }
}

/**
* Функция обраюотчик прерывания на кнопку питания 
*/
void intKey()
{
  int_key = true;
  detachInterrupt(0); //Отключаем прерывания 
}

/**
* Функия перевода ARDUINO в POWER_DOWN режим
*/
void EnterSleep()
{
  PowerOff();
  DebugPrint("I am sleeping");
  attachInterrupt(0, intKey, LOW); //Если на 0-вом прерываниии - ноль, то просыпаемся.
  power_on = false;
  delay(100);
  set_sleep_mode(SLEEP_MODE_PWR_DOWN); //Определяем режим сна
  
  sleep_enable(); //Разрешаем спящий режим
  sleep_mode(); //Спим (Прерывания продолжают работать.) Программа останавливается.
  sleep_disable(); //Запрещаем спящий режим
}

void DebugPrint( char *msg ){
   if( DebugFlag )Serial.println(msg);
} 

uint16_t l1=0,l2=0,l_min=0,l_max=0;
uint16_t u1 = 0;
double pulse = 0;
uint16_t l_mas[MAX_UNITS];


/**
* Текстовый экран
*/
void DisplayMode1(){ 
   display.clearDisplay();
   display.setTextSize(2);
   display.setCursor(0,0);
   display.print(l1);
   display.setTextSize(1);
   display.println(" Lx");
   display.println("");
   display.println("");
   display.print("Pulse: ");
   display.print(pulse,1);
   display.println("%");
   display.print("Bat: ");
   display.print(u1/1000);
   display.print(".");
   display.print((u1/100)%10);
   display.println(" V");
   display.display(); 
} 

/**
* Графический экран
*/
void DisplayMode2(){ 
   display.clearDisplay();
   display.setTextSize(1);
   display.setCursor(0,0);
   display.print(l1);
   display.print(" Lx. ");
   display.print(pulse,1);
   display.println("%");
 
   uint16_t dl = l_max - l_min;
   display.drawLine(0, 45, 84, 45 , BLACK);
   display.drawLine(78, 43, 84, 45 , BLACK);
   display.drawLine(78, 477, 84, 45 , BLACK);
   for( int i=0; i<MAX_UNITS; i++){
      uint16_t y = 44-(l_max-l_mas[i])*28/dl;
      display.drawLine(i, y, i, y-4, BLACK);
   }  
   
   display.display();
} 


 
void setup()
{
   if( DebugFlag ){
      Serial.begin(9600);
      Serial.println("Start luxmetr");
   }
   pinMode(13,OUTPUT);      // Светодиод питания
   pinMode(9,INPUT_PULLUP); // Выход на подсветку дисплея
   pinMode(8,INPUT_PULLUP); // Выход на питание дисплея

   analogRead(0);           // Порт для измерения напряжения питания по внутреннему источнику
   pinMode(2, INPUT_PULLUP);// Порт кнопки питания 
//




0=pinMode(10, INPUT_PULLUP);// Порт кнопки режима 
    attachInterrupt(0, intKey, LOW); //Если на 0-вом прерываниии - ноль, то просыпаемся.
   EnterSleep();            // Идем спать
 
}




 
unsigned long ms1 = 0; 
 
void loop()
{
   unsigned long ms = millis();
// Кнопка POWER нажата
   if( key_press ){
// Кнопка наконец то отаущена     
      if( digitalRead(2) == HIGH ){
         unsigned int key_interval = 0;
         if( DebugFlag ){
            Serial.print("Press key ");
            key_interval = abs(ms - key_time);
            Serial.print(key_count);
            Serial.print(" ");
            Serial.print(key_interval);
            Serial.println("");
         }
         key_count = 0;
         key_press = false;
         if( power_on && key_interval > 2000 ){
            DebugPrint("Power off");
            EnterSleep();
         }  
         else if( !power_on && key_interval < 2000 ){
            DebugPrint("Sleep again");
            EnterSleep();
         }
         else if( key_interval>2000 ){
           DebugPrint("Power on");
           power_on = true;
           PowerOn();
         }  
         else {
           DebugPrint("Press PWR key");
           if( key_mode2 == 1 ){key_mode2 = 2;DisplayMode2();}
           else {key_mode2 = 1;DisplayMode1();}
          }
      }  
   }
// Прерывание нажатия кнопки
   if( int_key ){
      delay(100);
      key_press = true;
// Первый тик времени нажатия кнопки
      if( key_count == 0 )key_time = ms;
      key_count++;
// Заряжаем обработку прерывния по новой
      int_key = false;
      attachInterrupt(0, intKey, LOW); //Если на 0-вом прерываниии - ноль, то просыпаемся.
   }

   
   
// Опрос датчиков каждую секунду
   if( (ms < ms1 || (ms - ms1) > 3000)&&power_on ){
      ms1 = ms;
      l1=0;l2=0;l_min=0;l_max=0;
      u1 = vccRead(100);
      uint32_t l_avg = 0;
      sensors_event_t event;
// Делаем MSX_UNITS измерений по 13мс      
      for( int i=0; i<MAX_UNITS; i++){
         tsl.getEvent(&event);
         l_avg += event.light;
         l_mas[i] = event.light;
         if( i == 0 ){
            l_min = event.light;
            l_max = event.light;
         }
         else {
            if( event.light < l_min )l_min = event.light;
            if( event.light > l_max )l_max = event.light;
         }
      } 
      l1 = l_avg / MAX_UNITS;
      if( l_max != 0 )pulse = (double)((l_max - l_min))*100/(double)l_max;
      else pulse = 0; 
      if( DebugFlag ){
         Serial.print("l_min(lx)=");
         Serial.print(l_min);
         Serial.print(",l_max(lx)=");
         Serial.print(l_max);
         Serial.print(",l1(lx)=");
         Serial.print(l1);
         Serial.print(",pulse(%)=");
         Serial.print(pulse,1);
         Serial.print(",Vcc (mV)=");
         Serial.println(u1);
     }

     if( key_mode2 == 1)DisplayMode1();
     else DisplayMode2();
   }
 
}
